part of snowm_scanner;

class TelemetryPacket {
  TelemetryPacket({
    this.rawData,
    this.uuid,
    this.battery,
  });

  String rawData, uuid = "TBD", macAddress;
  int battery = 0;

  static TelemetryPacket _parse(args) {
    String rawData = args["rawData"];
    TelemetryPacket packet = TelemetryPacket()..rawData = rawData;
    String battery = getBattery(rawData);
    packet.battery =
        canHaveBattery(rawData) ? convertHexToDecimal(battery) : null;
    packet.macAddress = getMacFromRAW(rawData);
    packet.uuid = asciiToHex(getUUIDFromMac(packet.macAddress));
    return packet;
  }

  StDevice toDevice() => StDevice(
        rawPacket: this.rawData,
        battery: this.battery.toString(),
        companyId: this.rawData.substring(10, 14),
        macAddress: this.macAddress,
        packetId: this.rawData.substring(26, 28),
        firmwareVersion: firmwareVersion(this.rawData),
        hardwareId: this.rawData.substring(34, 36),
        productSolutionId: this.rawData.substring(36, 38),
        airTemperatureRaw: this.rawData.substring(40, 44),
        surfaceTemperatureRaw: this.rawData.substring(44, 48),
        serialNo: this.rawData.substring(48),
      );
}

String _hexStringtoAscii(String hexString) {
  List<String> splitted = [];
  for (int i = 0; i < hexString.length; i = i + 2) {
    splitted.add(hexString.substring(i, i + 2));
  }
  String ascii = List.generate(splitted.length,
      (i) => String.fromCharCode(int.parse(splitted[i], radix: 16))).join();

  return ascii;
}

String asciiToHex(String value) {
  return value.characters.map((e) => e.codeUnitAt(0).toRadixString(16)).join();
}

String getBattery(String raw) {
  return raw.substring(38, 40);
}

int convertHexToDecimal(String str) {
  return int.parse(str, radix: 16);
}

String getUUIDFromMac(String mac) {
  return ("STSF" + mac).toUpperCase();
}

String getMacFromUUID(String uuid) {
  String ascii = _hexStringtoAscii(uuid);
  return ascii.replaceAll("STSF", "");
}

String getMacFromRAW(String raw) {
  return raw.substring(14, 26).toUpperCase();
}

String firmwareVersion(String raw) {
  return raw.substring(29, 34).replaceAll("0", ".");
}

bool canHaveBattery(String raw) {
  return firmwareVersion(raw).compareTo("1.3.0") >= 0;
}
